;  3D_MASS.LSP        3D MASS CALCULATOR          (C)2002, Matthew J. Dotson

;;;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
;;;@@	  3D Solid Object Weight Calculator  2002 Matthew J. Dotson (grimace74@wilkshire.net)
;;;@@	
;;;@@		This program was created to handle 3d solid object weight calculations dependent upon the objects
;;;@@  volume multiplied by a selected material density, or custom density.  The idea is to be able to theoretically 
;;;@@	find the weight of a given object drawn as a 3d solid model.  
;;;@@     To run this program simply type 3dmass at the command prompt or map a button on a toolbar to do so.	
;;;@@  Edited on May 14, 2019 by C.E. Hill for to write acadmass.mpr file to read/write path: C:/Users/chill/Documents/BCAD/acadmass.mpr
;;;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
;;; First we set up the dialog box and all of its options
(defun c:3dmass	()
  (setq	material '("Steel"	"Aluminum"   "Brass"	  "Cast Iron"
		   "Copper"	"Gold"	     "Lead"	  "Nickel"
		   "Platinum"	"Silver"     "Tin"	  "Titanium"	"Magnesium"
		  )
  )
					; list setup for materials used to set object density.

					;************Load Dialog Box******************
  (setq dcl_id (load_dialog "3D_Mass.dcl"))

					;****** Loads definitions in dcl file**********
  (if (not (new_dialog "mass" dcl_id)
					;test for dialog

      )					; not

    (exit)
					; exit if no dialog

  )					; end if

  (start_list "mats")
					; open the list box
  (mapcar 'add_list material)
					; add list of materials to listbox
  (end_list)
					; end the list box

  (set_tile "mats" "0")
  (set_tile "dens" "0.2833") 
  (action_tile
    "mats"
    "(setq matsuse $value)
  (if (= matsuse \"0\")
	(set_tile \"dens\" \"0.2833\"))
(if (= matsuse \"1\")
	(set_tile \"dens\" \"0.0975\"))
(if (= matsuse \"2\")
	(set_tile \"dens\" \"0.3033\"))
(if (= matsuse \"3\")
	(set_tile \"dens\" \"0.2665\"))
(if (= matsuse \"4\")
	(set_tile \"dens\" \"0.3210\"))
(if (= matsuse \"5\")
	(set_tile \"dens\" \"0.6969\"))
(if (= matsuse \"6\")
	(set_tile \"dens\" \"0.4096\"))
(if (= matsuse \"7\")
	(set_tile \"dens\" \"0.3178\"))
(if (= matsuse \"8\")
	(set_tile \"dens\" \"0.7717\"))
(if (= matsuse \"9\")
	(set_tile \"dens\" \"0.3780\"))
(if (= matsuse \"10\")
	(set_tile \"dens\" \"0.2633\"))
(if (= matsuse \"11\")
	(set_tile \"dens\" \"0.1621\"))
(if (= matsuse \"12\")
      (set_tile \"dens\" \"0.0628\"))" 
    
 )

(action_tile "dens"
  "(set_tile \"mats\" \"0\")"
)
  
(action_tile
    "cancel"

    "(done dialog) (setq runpick nil)"
					;close dialog

  )					; action_tile

  

  		
  (action_tile
    "picker"
    "(setq matdensity (atof (get_tile \"dens\")))
    (done_dialog)(setq runpick T)"
  )




  (start_dialog)
  (unload_dialog dcl_id)
					;unload


  (if runpick
    (progn
      
      (princ)
      (c:massmaker)
      (princ)
    )
  )
)


(defun c:massmaker ()
  (setq version (getvar "ACADVER"))
  (setq versnum (atof version))
  (setq solidset (ssget))
  (if (= solidset nil)
    (exit)
  )
  (setq l (sslength solidset))
  (setq j 0)
  (setq numsol 0)
  (while (/= j l)
    (setq solidgrp (entget (ssname solidset j)))
    (setq ename (cdr (assoc 0 solidgrp)))
    (if	(= ename "3DSOLID")
      (setq numsol (+ numsol 1))
    )
    (setq j (1+ j))
  )
  (if (= numsol 0)
    (progn
      (alert "\nNo 3D Solids Selected!")
      (exit)
    )
  )
  (command "massprop" solidset "" "y" "C:/Users/chill/Documents/BCAD/acadmass.mpr")
  
  (setq massfile (open "C:/Users/chill/Documents/BCAD/acadmass.mpr" "r"))
  (setq i 1)
  (while (not (= i 5))
    (setq massln (read-line massfile))
    (setq i (1+ i))
  )
  (setq objln (strlen massln))
  (setq masscheck (substr massln 26 (- objln 26)))
  (close massfile)
  (setq volumobj (atof masscheck))
  (setq masob (* volumobj matdensity))
  (setq massamt (rtos masob 2 3))
  (setq volamt (rtos volumobj 2 3))
  (setq densamt (rtos matdensity 2 3))
  (alert
    (strcat
      "The Mass of the selected Object(s) = " massamt " lbf"
      "\n     At a Density of " densamt " lbf/in"
    )
  )
;;; Delete the file created to read the Mass Properties
  (if (>= versnum 15.0)
    (progn
      (vl-file-delete "C:/Users/chill/Documents/BCAD/acadmass.mpr")
    )
    (progn
      (command "shell" (strcat "del \\acadmass" ".mpr"))
    )
  )

  (princ)
)


